/* id3.c

   ID3 reading and writing code
   By Matt Craven
   (c)2000 Hedgehog Software
*/

#include <stdio.h>
#include <string.h>
#include "id3.h"

/* Define the ID3 genre strings */

unsigned char *id3_genre_string[ID3_GENRE_MAX] = {
              "Blues", "Classic Rock", "Country",
              "Dance", "Disco", "Funk", "Grunge", "Hip-Hop", "Jazz",
              "Metal", "New Age", "Oldies", "Other", "Pop", "R&B", "Rap",
              "Reggae", "Rock", "Techno", "Industrial", "Alternative",
              "Ska", "Death Metal", "Pranks", "Soundtrack", "Euro-Techno",
              "Ambient", "Trip-Hop", "Vocal", "Jazz+Funk", "Fusion",
              "Trance", "Classical", "Instrumental", "Acid", "House", "Game",
              "Sound Clip", "Gospel", "Noise", "Alternative Rock", "Bass",
              "Soul", "Punk", "Space", "Meditative", "Instrumental Pop",
              "Instrumental Rock", "Ethnic", "Gothic", "Darkwave",
              "Techno-Industrial", "Electronic", "Pop-Folk", "Eurodance",
              "Dream", "Southern Rock", "Comedy", "Cult", "Gangsta", "Top 40",
              "Christian Rap", "Pop/Funk", "Jungle", "Native US", "Cabaret",
              "New Wave", "Psychadelic", "Rave", "Showtunes", "Trailer",
              "Lo-Fi", "Tribal", "Acid Punk", "Acid Jazz", "Polka", "Retro",
              "Musical", "Rock & Roll", "Hard Rock",

              /* now the additional winamp ones */

              "Folk", "Folk-Rock", "National Folk", "Swing", "Fast Fusion",
              "Bebop", "Latin", "Revival", "Celtic", "Bluegrass",
              "Avantgarde", "Gothic Rock", "Progressive Rock",
              "Psychadelic Rock", "Symphonic Rock", "Slow Rock", "Big Band",
              "Chorus", "Easy Listening", "Acoustic", "Humour", "Speech",
              "Chanson", "Opera", "Chamber Music", "Sonata", "Symphony",
              "Booty Bass", "Primus", "Porn Groove", "Satire",
              "Slow Jam", "Club", "Tabgo", "Samba", "Folklore",
              "Ballad", "Power Ballad", "Ryhtmic Soul", "Freestyle", "Duet",
              "Punk Rock", "Drum Solo","Acapella", "Euro-House",
              "Dance Hall", "Goa", "Drum & Bass", "Club-House", "Hardcore",
              "Terror", "Indie", "BritPop", "Negerpunk", "Polsk Punk",
              "Beat", "Christian Gangsta Rap", "Heavy Metal", "Black Metal",
              "Crossover", "Contemporary Christian", "Christian Rock",
              "Merengue", "Salsa", "Trash Metal", "Anime", "Jpop",
              "Synthpop" };

/*===================================================================
  id3_read(filename, tag)

  Gets the ID3 tag from the file given in "filename", storing tag
  info in "tag".

  If there was an ID3 tag it returns 1, otherwise it returns 0
  =================================================================*/

int id3_read(char *filename, ID3_TAG *infotag)
{
    char inbuf[256];
    FILE *fp;      
    int tagflag;
    int x;

    /* Initialise the tag to empty */

    for(x=0;x<30;x++) infotag->title[x] = ' ';
    for(x=0;x<30;x++) infotag->artist[x] = ' ';
    for(x=0;x<30;x++) infotag->album[x] = ' ';
    for(x=0;x<4;x++) infotag->year[x] = ' ';
    for(x=0;x<29;x++) infotag->comment[x] = ' ';

    /* Open the file */

    fp = fopen(filename,"rb");
    if(fp==NULL)
    {
       /* Quit if there was a problem */

       printf("Error opening file: %s\n",filename);    
       exit(-1);
    }

    /* look for ID tag (128 bytes from the end of the file) */

    if(fseek(fp,-128,SEEK_END))
    {
      /* Return if file smaller than 128 bytes */

      fclose(fp);
      return(0);
    }

    /* Read the (potential) ID3 tag */
    fread(inbuf,sizeof(char),128,fp);

    inbuf[129]=0;
    fclose(fp);

    /* check there actually is a tag: */

    tagflag=0;
    if( (inbuf[0]=='T' || inbuf[0]=='t')
    &&  (inbuf[1]=='A' || inbuf[1]=='a')
    &&  (inbuf[2]=='G' || inbuf[2]=='g'))
      tagflag=1;

    if(!tagflag)
    {
      /* there was no tag, so return */

      return(0);
    }

    /* Put the tag info into the tag structure */

    for(x=0;x<30;x++) infotag->title[x] = inbuf[3+x];
    for(x=0;x<30;x++) infotag->artist[x] = inbuf[33+x];
    for(x=0;x<30;x++) infotag->album[x] = inbuf[63+x];
    for(x=0;x<4;x++) infotag->year[x] = inbuf[93+x];
    for(x=0;x<29;x++) infotag->comment[x] = inbuf[97+x];
    infotag->track = inbuf[126];
    infotag->genre = inbuf[127];

    /* null-terminate the strings (in case we want to print them later */

    infotag->title[30]=0;
    infotag->artist[30]=0;
    infotag->album[30]=0;
    infotag->year[4]=0;
    infotag->comment[29]=0;

    /* Return the fact that there was an ID3 tag */

    return 1;
}

/*===================================================================
  id3_tag_to_string(tag, txt)

  Converts the given ID3 "tag" to a 128-character string, storing it
  in "txt"
  =================================================================*/

void id3_tag_to_string(ID3_TAG tag, char *txt)
{
  int x;

  /* Initialise the string */

  for(x=0;x<128;x++)
     txt[x] = 32;

  /* TAG, title, artist, album, year, comment, track, genre */

  txt[0] = 'T'; txt[1] = 'A'; txt[2] = 'G';

  for(x=0;x<30;x++) txt[3+x] = tag.title[x];
  for(x=0;x<30;x++) txt[33+x] = tag.artist[x];
  for(x=0;x<30;x++) txt[63+x] = tag.album[x];
  for(x=0;x<4;x++) txt[93+x] = tag.year[x];
  for(x=0;x<29;x++) txt[97+x] = tag.comment[x];
  txt[126] = tag.track;
  txt[127] = tag.genre;
}

/*===================================================================
  id3_write(filename, tag)

  Writes the given tag onto the given file (replacing an existing
  tag if there was one)
  =================================================================*/

int id3_write(char *filename, ID3_TAG tag)
{
   int tagged;
   ID3_TAG dummy;
   FILE *fp;
   char id3_string[128];

   /* See if it's already tagged */

   tagged = id3_read(filename, &dummy);

   /* Open the file for binary append */

   fp=fopen(filename,"ab");

   /* Return if something went wrong */

   if(fp==NULL)
      return 0;

   /* If it's already tagged, seek back 128 bytes from end of file */

   if(tagged)
   {
     if(fseek(fp,-128,SEEK_END))
     {
       fclose(fp);
       return 0;
     }
   }

   /* convert the tag to a string */

   id3_tag_to_string(tag, id3_string);

   /* write the tag */

   fwrite(id3_string,sizeof(char),128,fp);

   fclose(fp);

   /* say we succeeded */

   return 1;
}

